//
//  UIColor+Category.h
//  RippleImageView
//
//  Created by LY on 2016/12/9.
//  Copyright © 2016年 LY. All rights reserved.
//

#import <UIKit/UIKit.h>

@interface UIColor (Category)

/** 两个颜色是否相同?(相近 8bit) */
- (BOOL)isEqualToColor:(UIColor *)color;

/** 两个颜色是否相似 用RGB比较 */
- (BOOL)isApproximateToColorRGB:(UIColor *)color precision:(CGFloat)range;

/** 两个颜色是否相似 用HSV比较*/
- (BOOL)isApproximateToColorHSV:(UIColor *)color precision:(CGFloat)radius;

/** 互补色 */
- (UIColor *)complementaryColorRGB;
- (UIColor *)complementaryColorHSV;

/** 构造颜色 */
+ (UIColor *)colorWithHexRGB:(UInt32)rgb;   // 低24bit有效
+ (UIColor *)colorWithHexRGBA:(UInt32)rgba; // 32bit有效

+ (UIColor *)colorWithHexHSV:(UInt32)hsv;   // 低24bit有效
+ (UIColor *)colorWithHexHSVA:(UInt32)hsva; // 32bit有效

+ (UIColor *)colorHexR:(UInt8)r g:(UInt8)g b:(UInt8)b;
+ (UIColor *)colorHexR:(UInt8)r g:(UInt8)g b:(UInt8)b a:(UInt8)a;

+ (UIColor *)colorHexH:(UInt8)h s:(UInt8)s v:(UInt8)v;
+ (UIColor *)colorHexH:(UInt8)h s:(UInt8)s v:(UInt8)v a:(UInt8)a;

+ (UIColor *)colorFloatR:(CGFloat)r g:(CGFloat)g b:(CGFloat)b;
+ (UIColor *)colorFloatH:(CGFloat)h s:(CGFloat)s v:(CGFloat)v;


/** 随机颜色 */
+ (UIColor *)randomColor;


/** UIColor转16进制RGBA */
- (UInt32)hexRGBA;

/** UIColor转16进制HSVA */
- (UInt32)hexHSVA;



/** red */
- (CGFloat)redFloat;
- (UInt8)redHex;

/** green */
- (CGFloat)greenFloat;
- (UInt8)greenHex;

/** blue */
- (CGFloat)blueFloat;
- (UInt8)blueHex;



/** hue */
- (CGFloat)hueFloat;
- (UInt8)hueHex;

/** saturation */
- (CGFloat)saturationFloat;
- (UInt8)saturationHex;

/** brightness */
- (CGFloat)brightnessFloat;
- (UInt8)brightnessHex;



/** alpha */
- (CGFloat)alphaFloat;
- (UInt8)alphaHex;

/** 显示rgba */
- (void)displayRGBA;

/** 显示hsvb */
- (void)displayHSVB;

@end
