//
//  UIColor+Category.m
//  RippleImageView
//
//  Created by LY on 2016/12/9.
//  Copyright © 2016年 LY. All rights reserved.
//

#import "UIColor+Category.h"

#define floatToHex(f, b)    (((UInt8)((f) * 255)) << ((b) * 8))

#define hex32_8(h, i)       (((h) >> ((i) * 8)) & 0xFF)

#define uint8cvt(x)         ((x) & 0xff)

#define cs0_1(f)            (MAX(0, MIN(1, (f))))

@implementation UIColor (Category)

- (BOOL)isEqualToColor:(UIColor *)color
{
    UInt32 rgba0 = [self hexRGBA];
    UInt32 rgba1 = [color hexRGBA];
    return rgba0 == rgba1;
}

- (BOOL)isApproximateToColorRGB:(UIColor *)color precision:(CGFloat)range
{
    CGFloat r0, g0, b0, a0, r1, g1, b1, a1;
    [self getRed:&r0 green:&g0 blue:&b0 alpha:&a0];
    [color getRed:&r1 green:&g1 blue:&b1 alpha:&a1];
    CGFloat l = sqrt(pow(r0 - r1, 2) + pow(g0 - g1, 2) + pow(b0 - b1, 2));
    //    l = MAX(MAX(fabs(r0 - r1), fabs(g0 - g1)), fabs(b0 - b1));
    //    NSLog(@"r:%f g:%f b:%f", r0, g0, b0);
    //    NSLog(@"r:%f g:%f b:%f", r1, g1, b1);
    //    NSLog(@"l:%f", l);
    return l <= range;
}

- (BOOL)isApproximateToColorHSV:(UIColor *)color precision:(CGFloat)radius
{
    CGFloat r0, g0, b0, a0, r1, g1, b1, a1;
    [self getHue:&r0 saturation:&g0 brightness:&b0 alpha:&a0];
    [color getHue:&r1 saturation:&g1 brightness:&b1 alpha:&a1];
    CGFloat l = sqrt(pow(r0 - r1, 2) + pow(g0 - g1, 2) + pow(b0 - b1, 2));
    //    NSLog(@"r:%f g:%f b:%f", r0, g0, b0);
    //    NSLog(@"r:%f g:%f b:%f", r1, g1, b1);
    //    NSLog(@"l:%f", l);
    return l <= radius;
}

- (UIColor *)complementaryColorRGB
{
    UInt32 c0 = [self hexRGBA] >> 8;
    UInt32 c1 = c0 ^ 0xFFFFFF;
    return [UIColor colorWithHexRGB:c1];
}

- (UIColor *)complementaryColorHSV
{
    UInt32 c0 = [self hexHSVA] >> 8;
    UInt32 c1 = c0 ^ 0xFFFFFF;
    return [UIColor colorWithHexRGB:c1];
}

+ (UIColor *)colorWithHexRGB:(UInt32)rgb
{
    UInt8 r = hex32_8(rgb, 2), g = hex32_8(rgb, 1), b = hex32_8(rgb, 0);
    return [self colorHexR:r g:g b:b];
}

+ (UIColor *)colorWithHexRGBA:(UInt32)rgba
{
    UInt8 r = hex32_8(rgba, 3), g = hex32_8(rgba, 2), b = hex32_8(rgba, 1), a = hex32_8(rgba, 0);
    return [self colorHexR:r g:g b:b a:a];
}

+ (UIColor *)colorHexR:(UInt8)r g:(UInt8)g b:(UInt8)b
{
    return [self colorHexR:r g:g b:b a:0xff];
}

+ (UIColor *)colorHexR:(UInt8)r g:(UInt8)g b:(UInt8)b a:(UInt8)a
{
    return [UIColor colorWithRed:uint8cvt(r) / 255.0 green:uint8cvt(g) / 255.0 blue:uint8cvt(b) / 255.0 alpha:uint8cvt(a) / 255.0];
}

+ (UIColor *)colorWithHexHSV:(UInt32)hsv
{
    UInt8 h = hex32_8(hsv, 2), s = hex32_8(hsv, 1), v = hex32_8(hsv, 0);
    return [self colorHexH:h s:s v:v];
}

+ (UIColor *)colorWithHexHSVA:(UInt32)hsva
{
    UInt8 h = hex32_8(hsva, 3), s = hex32_8(hsva, 2), v = hex32_8(hsva, 1), a = hex32_8(hsva, 0);
    return [self colorHexH:h s:s v:v a:a];
}

+ (UIColor *)colorHexH:(UInt8)h s:(UInt8)s v:(UInt8)v
{
    return [self colorHexH:h s:s v:v a:0xff];
}

+ (UIColor *)colorHexH:(UInt8)h s:(UInt8)s v:(UInt8)v a:(UInt8)a
{
    return [UIColor colorWithHue:uint8cvt(h) / 255.0 saturation:uint8cvt(s) / 255.0 brightness:uint8cvt(v) / 255.0 alpha:uint8cvt(a) / 255.0];
}

+ (UIColor *)colorFloatR:(CGFloat)r g:(CGFloat)g b:(CGFloat)b
{
    return [self colorWithRed:cs0_1(r) green:cs0_1(g) blue:cs0_1(b) alpha:1];
}

+ (UIColor *)colorFloatH:(CGFloat)h s:(CGFloat)s v:(CGFloat)v
{
    return [self colorWithHue:cs0_1(h) saturation:cs0_1(s) brightness:cs0_1(v) alpha:1];
}

+ (UIColor *)randomColor
{
    return [UIColor colorHexR:arc4random() % 255 g:arc4random() % 255 b:arc4random() % 255 a:arc4random() % 255];
}

- (UInt32)hexRGBA
{
    CGFloat r, g, b, a;
    [self getRed:&r green:&g blue:&b alpha:&a];
    UInt32 hex = floatToHex(r, 3) + floatToHex(g, 2) + floatToHex(b, 1) + floatToHex(a, 0);
    return hex;
}

- (UInt32)hexHSVA
{
    CGFloat h, s, v, a;
    [self getHue:&h saturation:&s brightness:&v alpha:&a];
    UInt32 hex = floatToHex(h, 3) + floatToHex(s, 2) + floatToHex(v, 1) + floatToHex(a, 0);
    return hex;
}

/** red */
- (CGFloat)redFloat
{
    CGFloat r, g, b, a;
    [self getRed:&r green:&g blue:&b alpha:&a];
    return r;
}

- (UInt8)redHex
{
    UInt32 rgba = [self hexRGBA];
    return hex32_8(rgba, 3);
}

/** green */
- (CGFloat)greenFloat
{
    CGFloat r, g, b, a;
    [self getRed:&r green:&g blue:&b alpha:&a];
    return g;
}

- (UInt8)greenHex
{
    UInt32 rgba = [self hexRGBA];
    return hex32_8(rgba, 2);
}


/** blue */
- (CGFloat)blueFloat
{
    CGFloat r, g, b, a;
    [self getRed:&r green:&g blue:&b alpha:&a];
    return b;
}

- (UInt8)blueHex
{
    UInt32 rgba = [self hexRGBA];
    return hex32_8(rgba, 1);
}


/** hue */
- (CGFloat)hueFloat
{
    CGFloat h, s, v, a;
    [self getHue:&h saturation:&s brightness:&v alpha:&a];
    return h;
}

- (UInt8)hueHex
{
    UInt32 hsva = [self hexHSVA];
    return hex32_8(hsva, 3);
}


/** saturation */
- (CGFloat)saturationFloat
{
    CGFloat h, s, v, a;
    [self getHue:&h saturation:&s brightness:&v alpha:&a];
    return s;
}

- (UInt8)saturationHex
{
    UInt32 hsva = [self hexHSVA];
    return hex32_8(hsva, 2);
}


/** brightness */
- (CGFloat)brightnessFloat
{
    CGFloat h, s, v, a;
    [self getHue:&h saturation:&s brightness:&v alpha:&a];
    return v;
}

- (UInt8)brightnessHex
{
    UInt32 hsva = [self hexHSVA];
    return hex32_8(hsva, 1);
}



/** alpha */
- (CGFloat)alphaFloat
{
    CGFloat r, g, b, a;
    [self getRed:&r green:&g blue:&b alpha:&a];
    return a;
}

- (UInt8)alphaHex
{
    UInt32 rgba = [self hexRGBA];
    return hex32_8(rgba, 0);
}

- (void)displayRGBA
{
    CGFloat r, g, b, a;
    [self getRed:&r green:&g blue:&b alpha:&a];
    NSLog(@"color r:%f g:%f b:%f a:%f", r, g, b, a);
}

- (void)displayHSVB
{
    CGFloat h, s, v, a;
    [self getHue:&h saturation:&s brightness:&v alpha:&a];
    NSLog(@"color h:%f s:%f v:%f a:%f", h, s, v, a);
}

@end
