//
//  UIImage+Category.h
//  RippleImageView
//
//  Created by LY on 2016/12/9.
//  Copyright © 2016年 LY. All rights reserved.
//

#import <UIKit/UIKit.h>

typedef NS_ENUM(NSUInteger, ImageRotatingTo) {
    ImageRotatingTo_Left,       // 向左旋转90度
    ImageRotatingTo_Right,      // 向右旋转90度
    ImageRotatingTo_UpsideDown  // 旋转180度
};

@interface UIImage (Category)

/** 获取位图尺寸  图片实际的像素宽高 */
@property (nonatomic, readonly)NSInteger pixw;
@property (nonatomic, readonly)NSInteger pixh;

/** 纯色image 1x1像素 */
+ (UIImage *)imageWithColor:(UIColor *)color;

/** 纯色image size像素 */
+ (UIImage *)imageWithColor:(UIColor *)color size:(CGSize)size;

/** 纯色image size像素 带圆角 */
+ (UIImage *)imageWithColor:(UIColor *)color size:(CGSize)size radius:(CGFloat)radius;

/** 图片压缩 size:压缩后尺寸 */
+ (UIImage *)imageWithImageSimple:(UIImage*)image scaledToSize:(CGSize)newSize;

/** 旋转图片 orientation:旋转角度 ( 宽高可能会变 ) */
- (UIImage *)rotatingWithRotation:(ImageRotatingTo)orientation;
@end




#pragma mark - Image_Bitmap
@interface UIImage (Image_Bitmap)

// 创建bitmap
- (CGContextRef)createRGBBitmapContext;

// 获取位图数据 bitmap data
- (unsigned char *)bitmapData;

// 由data生成image
- (UIImage *)imageWithBitMapData:(void *)data;

// 由data生成image
+ (UIImage *)imageWithBitMapData:(void *)data size:(CGSize)size;

// 由context生成image
+ (UIImage *)imageWithContext:(CGContextRef)context size:(CGSize)size;

@end
