//
//  UIImage+Category.m
//  RippleImageView
//
//  Created by LY on 2016/12/9.
//  Copyright © 2016年 LY. All rights reserved.
//

#import "UIImage+Category.h"
#import "UIView+Category.h"

#define kSingleLineWidth            (1 / [UIScreen mainScreen].scale)
#define safe8bit(i)                 MIN(255, MAX(0, (i)))

#define _nflt(i)    ( [NSNumber numberWithFloat:(i)] )

#define _n273(i)    ( [NSNumber numberWithFloat:(i) / 273.0] )

@interface GetImageView : UIView

@property (nonatomic, assign) CGFloat radius;
@property (nonatomic, strong) UIColor *fillcolor;

@end

@implementation GetImageView


- (GetImageView *)initWithFrame:(CGRect)frame cornerRadius:(CGFloat)radius fillcolor:(UIColor *)color
{
    self = [super initWithFrame:frame];
    if (self) {
        self.fillcolor = color;
        self.radius = radius;
        self.backgroundColor = self.fillcolor;
        self.layer.borderWidth = kSingleLineWidth;
        self.layer.borderColor = self.fillcolor.CGColor;
        self.layer.cornerRadius = self.radius;
        self.layer.masksToBounds = YES;
    }
    return self;
}

- (GetImageView *)initWithFrame:(CGRect)frame cornerRadius:(CGFloat)radius fillcolor:(UIColor *)color borderWidth:(CGFloat)borderWidth borderColor:(UIColor *)borderColor
{
    self = [super initWithFrame:frame];
    if (self) {
        self.fillcolor = color;
        self.radius = radius;
        self.backgroundColor = self.fillcolor;
        self.layer.borderWidth = borderWidth;
        self.layer.borderColor = borderColor.CGColor;
        self.layer.cornerRadius = self.radius;
        self.layer.masksToBounds = YES;
    }
    return self;
}
@end


@implementation UIImage (Category)

- (NSInteger)pixw
{
    size_t w = CGImageGetWidth(self.CGImage);
    return w;
}

- (NSInteger)pixh
{
    size_t h = CGImageGetHeight(self.CGImage);
    return h;
}

+ (UIImage *)imageWithColor:(UIColor *)color
{
    GetImageView *view = [[GetImageView alloc] initWithFrame:CGRectMake(0, 0, 1, 1) cornerRadius:0 fillcolor:color];
    UIImage *image = [view screenShot];
    return image;
}

+ (UIImage *)imageWithColor:(UIColor *)color size:(CGSize)size
{
    GetImageView *view = [[GetImageView alloc] initWithFrame:CGRectMake(0, 0, size.width, size.height) cornerRadius:0 fillcolor:color];
    UIImage *image = [view screenShot];
    return image;
}

+ (UIImage *)imageWithColor:(UIColor *)color size:(CGSize)size radius:(CGFloat)radius
{
    GetImageView *view = [[GetImageView alloc] initWithFrame:CGRectMake(0, 0, size.width, size.height) cornerRadius:radius fillcolor:color];
    UIImage *image = [view screenShot];
    return image;
}

+ (UIImage*)imageWithImageSimple:(UIImage*)image scaledToSize:(CGSize)newSize
{
    UIGraphicsBeginImageContext(newSize);
    [image drawInRect:CGRectMake(0,0,newSize.width,newSize.height)];
    UIImage* newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

- (UIImage *)turnToOrientation:(UIImageOrientation)orientation
{
    UIImage *image = [UIImage imageWithCGImage:self.CGImage scale:1 orientation:orientation];
    return image;
}

- (UIImage *)rotatingWithRotation:(ImageRotatingTo)orientation
{
    // 修正方向
    UIImage *image = [self turnToOrientation:UIImageOrientationUp];
    CGFloat rotate = 0.0;
    CGRect rect;
    float translateX = 0;
    float translateY = 0;
    float scaleX = 1.0;
    float scaleY = 1.0;
    
    switch (orientation)
    {
        case ImageRotatingTo_Left:
        {
            rotate = M_PI_2;
            rect = CGRectMake(0, 0, image.size.height, image.size.width);
            translateX = 0;
            translateY = -rect.size.width;
            scaleY = rect.size.width/rect.size.height;
            scaleX = rect.size.height/rect.size.width;
            break;
        }
        case ImageRotatingTo_Right:
        {
            rotate = 3 * M_PI_2;
            rect = CGRectMake(0, 0, image.size.height, image.size.width);
            translateX = -rect.size.height;
            translateY = 0;
            scaleY = rect.size.width/rect.size.height;
            scaleX = rect.size.height/rect.size.width;
            break;
        }
        case ImageRotatingTo_UpsideDown:
        {
            rotate = M_PI;
            rect = CGRectMake(0, 0, image.size.width, image.size.height);
            translateX = -rect.size.width;
            translateY = -rect.size.height;
            break;
        }
        default:
        {
            rotate = 0.0;
            rect = CGRectMake(0, 0, image.size.width, image.size.height);
            translateX = 0;
            translateY = 0;
            break;
        }
    }
    
    UIGraphicsBeginImageContext(rect.size);
    CGContextRef context = UIGraphicsGetCurrentContext();
    //做CTM变换
    CGContextTranslateCTM(context, 0.0, rect.size.height);
    CGContextScaleCTM(context, 1.0, -1.0);
    CGContextRotateCTM(context, rotate);
    CGContextTranslateCTM(context, translateX, translateY);
    CGContextScaleCTM(context, scaleX, scaleY);
    
    //绘制图片
    CGContextDrawImage(context, CGRectMake(0, 0, rect.size.width, rect.size.height), image.CGImage);
    UIImage *newPic = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newPic;
}

@end

@implementation UIImage (Image_Bitmap)

//创建bitmap
- (CGContextRef)createRGBBitmapContext
{
    NSInteger w = [self pixw], h = [self pixh];
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    if (colorSpace == NULL)
    {
        fprintf(stderr, "Error allocating color space\n");
        return NULL;
    }
    void *bitmapData = malloc(4 * w * h);
    if (bitmapData == NULL)
    {
        fprintf (stderr, "Memory not allocated!");
        CGColorSpaceRelease(colorSpace);
        return NULL;
    }
    CGContextRef context = CGBitmapContextCreate(bitmapData, w, h, 8, w * 4,colorSpace,2);//2 == kCGImageAlphaPremultipliedFirst;
    CGColorSpaceRelease(colorSpace);
    if (context == NULL)
    {
        free (bitmapData);
        fprintf (stderr, "Context not created!");
    }
    return context;
}

// 获取位图数据 bitmap data
- (unsigned char *)bitmapData
{
    CGContextRef context = [self createRGBBitmapContext];
    if (context == NULL) return NULL;
    NSInteger w = [self pixw], h = [self pixh];
    CGRect rect = {{0, 0}, {w, h}};
    CGContextDrawImage(context, rect, self.CGImage);
    unsigned char* data = CGBitmapContextGetData(context);
    CGContextRelease(context);
    if (data == NULL) return NULL;
    return data;
}

// 由data生成image
- (UIImage *)imageWithBitMapData:(void *)data
{
    NSInteger w = [self pixw];
    NSInteger h = [self pixh];
    return [UIImage imageWithBitMapData:data size:CGSizeMake(w, h)];
}


// 由data生成image
+ (UIImage *)imageWithBitMapData:(void *)data size:(CGSize)size
{
    NSInteger w = size.width;
    NSInteger h = size.height;
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGContextRef context = CGBitmapContextCreate(data, w, h, 8, w * 4, colorSpace, 2);
    //2 == kCGImageAlphaPremultipliedFirst
    CGImageRef imgref = CGBitmapContextCreateImage(context);
    CGColorSpaceRelease(colorSpace);
    CGContextRelease(context);
    UIImage *image = [UIImage imageWithCGImage:imgref];
    CFRelease(imgref);
    return image;
}

// 由context生成image
+ (UIImage *)imageWithContext:(CGContextRef)context size:(CGSize)size
{
    if (context == NULL) return nil;
    CGContextScaleCTM(context, 1, -1);
    CGContextTranslateCTM(context, 0, -size.height);
    CGImageRef img = CGBitmapContextCreateImage(context);
    UIImage* image = [UIImage imageWithCGImage:img];
    CGImageRelease(img);
    CGContextRelease(context);
    return image;
}

@end

