//
//  UIImageView+Category.h
//  RippleImageView
//
//  Created by LY on 2016/12/9.
//  Copyright © 2016年 LY. All rights reserved.
//

#import <UIKit/UIKit.h>


@interface RippleImageBitMap : NSObject

@property (nonatomic, strong) UIImage *image;           // 波纹位图填充image
@property (nonatomic, strong) UIImage *replaceImage;    // 波纹位图代替image
@property (nonatomic, assign) CGFloat replaceRadius;    // 波纹位图代替半径

/**
 * 图片生成波纹效果
 * @param
 *  point       生成波纹的中心点(0~1) (0.5, 0.5)表示图片中心
 *  wavelength  波长
 *  amplitude   振幅
 *  phase       偏移量 可依此生成动画
 *  radius      半径  波纹半径
 */
- (UIImage *)rippleImageAtPoint:(CGPoint)point wavelength:(CGFloat)wavelength amplitude:(CGFloat)amplitude phase:(CGFloat)phase raduis:(CGFloat)radius;
@end


@interface UIImageView (Category)
/**
 * 波纹动画
 *  波纹生成中心点 视图中心点(默认)     如果之前有中心点值,使用之前的
 *  产生波纹的半径 短边的一半(默认)     如果之前有半径值,使用之前的
 *  波长              4   (默认)     如果之前有波长值,使用之前的
 *  振幅              15  (默认)     如果之前有振幅值,使用之前的
 *  扩散速度          10   (默认)     如果之前有速度值,使用之前的
 * @p block     停止控制回调 (返回YES停止动画)
 */
- (void)startRippleAnimation:(BOOL (^)(void))block;

/**
 * 波纹动画
 *  @p point         波纹生成中心点 (0~1) 例:(0.5, 0.5)表示视图中心点
 *  @p radius        产生波纹的半径
 *  波长              4   (默认)     如果之前有波长值,使用之前的
 *  振幅              15  (默认)     如果之前有振幅值,使用之前的
 *  扩散速度          10   (默认)     如果之前有速度值,使用之前的
 *  @p block     停止控制回调 (返回YES停止动画)
 */
- (void)startRippleAnimationAtPoint:(CGPoint)point radius:(CGFloat)radius callback:(BOOL (^)(void))block;

/**
 * 波纹动画
 *  @p point         波纹生成中心点 (0~1) 例:(0.5, 0.5)表示视图中心点
 *  @p radius        产生波纹的半径
 *  @p wavelength    波长
 *  @p amplitude     振幅
 *  扩散速度          10    (默认) 如果之前有速度值,使用之前的
 * @p block         停止控制回调 (返回YES停止动画)
 */
- (void)startRippleAnimationAtPoint:(CGPoint)point radius:(CGFloat)radius wavelength:(CGFloat)wavelength amplitude:(CGFloat)amplitude callback:(BOOL (^)(void))block;

/**
 * 波纹动画
 * @p point         波纹生成中心点 (0~1) 例:(0.5, 0.5)表示视图中心点
 * @p radius        产生波纹的半径
 * @p wavelength    波长
 * @p amplitude     振幅
 * @p speed         波纹运动速度 (0~60) 越大扩展越快 0停止
 * @p block         停止控制回调 (返回YES停止动画)
 */
- (void)startRippleAnimationAtPoint:(CGPoint)point radius:(CGFloat)radius wavelength:(CGFloat)wavelength amplitude:(CGFloat)amplitude speed:(CGFloat)speed callback:(BOOL (^)(void))block;


/** 手动停止波纹动画 */
- (void)stopRippleAnimation;

/** 重启动画 - 只针对暂停 */
- (void)restartRippleAnimation;

/** 暂停动画 */
- (void)suspendRippleAnimation;


/**  单独设置参数 只有开启波纹动画之后才有意义 (基于图片的像素, 而不是视图的尺寸) */
- (void)resetRippleRadius:(CGFloat)radius;
- (void)resetRippleWavelength:(CGFloat)wavelength;
- (void)resetRippleAmplitude:(CGFloat)amplitude;
- (void)resetRippleSpeed:(CGFloat)speed;
- (void)resetRippleSlope_dx:(CGFloat)dx dy:(CGFloat)dy;// [1 <= dx <= 10] [1 <= dy <= 10]
- (void)resetRippleCenter:(CGPoint)center;
- (void)resetRippleImage:(UIImage *)image;


- (void)resetRippleReplaceImage:(UIImage *)image; // 替换位图
- (void)resetRippleReplaceRadius:(CGFloat)radius; // 替换半径

@end
