//
//  UIImageView+Category.m
//  RippleImageView
//
//  Created by LY on 2016/12/9.
//  Copyright © 2016年 LY. All rights reserved.
//

#import "UIImageView+Category.h"
#import "UIImage+Category.h"
#import <objc/runtime.h>
#import "CG_Extention.h"
#import "NSObject+Category.h"
#import "UIView+Category.h"


typedef struct {
    UInt32 nw, ne;
    UInt32 sw, se;
}PointRect;

/** 根据参数生成波纹image */
@interface RippleImageBitMap ()

// 创建位图失败则不能生成波纹图
@property (nonatomic, assign) BOOL available;

@end

@implementation RippleImageBitMap
{
    // 需要多次调用
    UInt32 *imageBitmapData;
    UInt32 *createBitmapData;
    UInt32 *replaceBitmapData;
}

- (UIImage *)rippleImageAtPoint:(CGPoint)point wavelength:(CGFloat)wavelength amplitude:(CGFloat)amplitude phase:(CGFloat)phase raduis:(CGFloat)radius
{
    return [self rippleImageAtPoint:point wavelength:wavelength amplitude:amplitude phase:phase raduis:radius dx:1 dy:1];
}


- (UIImage *)rippleImageAtPoint:(CGPoint)point wavelength:(CGFloat)wavelength amplitude:(CGFloat)amplitude phase:(CGFloat)phase raduis:(CGFloat)radius dx:(CGFloat)dx dy:(CGFloat)dy
{
    if (!self.available)
    {
        return self.image;
    }
    NSInteger width = [self.image pixw];
    NSInteger height = [self.image pixh];
    
    CGPoint center = CGPointMake(width * point.x, height * point.y);
    radius = (radius == 0) ? MIN(center.x, center.y) : radius;
    radius = radius * radius;
    
    for(int row = 0; row < height; row++)
    {
        for(int col = 0; col < width; col++)
        {
            
            
            CGFloat d_x = col - center.x;
            CGFloat d_y = row - center.y;
            CGFloat distance2 = dx * d_x * d_x + dy * d_y * d_y;
            
            if (distance2 < self.replaceRadius)
            {
                createBitmapData[row * width + col] = replaceBitmapData[row * width + col];
                continue;
            }
            
            // 获取水波的扩散位置，最重要的一步
            CGPoint thePoint = [self generateWaterRipplesX:col y:row center:center amplitude:amplitude wavelength:wavelength phase:phase radius:radius dx:dx dy:dy];
            NSInteger srcX = (int)floor(thePoint.x);
            NSInteger srcY = (int)floor(thePoint.y);
            
            // 获取周围四个像素，插值用
            PointRect pr = [self calculateIndex:width * srcY + srcX w:width h:height];
            // 取得对应的振幅位置P(x, y)的像素，使用双线性插值
            createBitmapData[row * width + col] = [self bilinearInterpolateX:thePoint.x - srcX y:thePoint.y - srcY nw:pr.nw ne:pr.ne sw:pr.sw se:pr.se];
        }
    }
    UIImage *ripple = [self.image imageWithBitMapData:createBitmapData];
    return ripple;
}

// 获取水波的扩散位置
- (CGPoint)generateWaterRipplesX:(NSInteger)x y:(NSInteger)y center:(CGPoint)center amplitude:(CGFloat)amplitude wavelength:(CGFloat)wavelength phase:(CGFloat)phase radius:(CGFloat)radius dx:(CGFloat)d_x dy:(CGFloat)d_y
{
    CGPoint pot = CGPointZero;
    CGFloat dx = x - center.x;
    CGFloat dy = y - center.y;
    CGFloat distance2 = dx * dx * d_x + dy * dy * d_y;
    // 确定 water ripple的半径，如果在半径之外，就直接获取原来位置，不用计算迁移量
    if (distance2 > radius)
    {
        pot.x = x;
        pot.y = y;
    }
    else
    {
        // 如果在radius半径之内，计算出来
        float distance = (float)sqrt(distance2);
        
        float t = (float)sqrt(distance2 / radius);
        float base = 1 - t * t;
        // 振幅因子 距波源越远 因子越小
        float emp = base;//(0.2 + 1 - (0.8 * distance / (float)sqrt(radius)));
        // 计算改点振幅
        float amount = emp * amplitude * (float)sin(distance / wavelength * M_2_PI - phase);
        // 计算能量损失，
        amount *= (radius - distance) / radius;
        if ( distance != 0 ) amount *= wavelength / distance;
        // 得到water ripple 最终迁移位置
        pot.x = x + dx * amount;
        pot.y = y + dy * amount;
    }
    return pot;
}

// 优化边缘算法
- (PointRect)calculateIndex:(NSInteger)index w:(NSInteger)w h:(NSInteger)h
{
    PointRect pr = {0, 0, 0, 0};
    NSInteger i = index;
    // 限制在原bitmap和+1倍bitmap
    if (i < 0)
    {
        i = -i;
    }
    if (i > w * h * 2 - 1)
    {
        i = i - w * h * 2;
    }
    
    // 重新计算取点位置 边缘检测
    // 左上边缘正常处理, 最右点则往左回溯, 最下点则往上回溯, 超过原bitmap折叠计算
    NSInteger offrow = 1, offline = w;
    NSInteger line = i / w, row = i % w;
    if (row == w - 1)
    {
        offrow = -1;
    }
    if (line >= h - 1)
    {
        if (line <= h)
        {
            offline = -w;
        }
        if (line >= h)//超过最大行
        {
            i = i - ((line - h) * 2 + 1) * w;
        }
    }
    pr.nw = imageBitmapData[i];
    pr.ne = imageBitmapData[i + offrow];
    pr.sw = imageBitmapData[i + offline];
    pr.se = imageBitmapData[i + offline + offrow];
    return pr;
}

// 双线性插值
- (UInt32)bilinearInterpolateX:(CGFloat)x y:(CGFloat)y nw:(UInt32)nw ne:(UInt32)ne sw:(UInt32)sw se:(UInt32)se
{
    float m0, m1;
    int a0 = (nw >> 24) & 0xff; //a->b
    int r0 = (nw >> 16) & 0xff; //r->g
    int g0 = (nw >>  8) & 0xff; //g->r
    int b0 = nw & 0xff;         //b->a
    
    int a1 = (ne >> 24) & 0xff;
    int r1 = (ne >> 16) & 0xff;
    int g1 = (ne >>  8) & 0xff;
    int b1 = ne & 0xff;
    
    int a2 = (sw >> 24) & 0xff;
    int r2 = (sw >> 16) & 0xff;
    int g2 = (sw >>  8) & 0xff;
    int b2 = sw & 0xff;
    
    int a3 = (se >> 24) & 0xff;
    int r3 = (se >> 16) & 0xff;
    int g3 = (se >>  8) & 0xff;
    int b3 = se & 0xff;
    
    float cx = 1.0f - x;
    float cy = 1.0f - y;
    
    m0 = cx * a0 + x * a1;
    m1 = cx * a2 + x * a3;
    int a = (int)(cy * m0 + y * m1);
    
    m0 = cx * r0 + x * r1;
    m1 = cx * r2 + x * r3;
    int r = (int)(cy * m0 + y * m1);
    
    m0 = cx * g0 + x * g1;
    m1 = cx * g2 + x * g3;
    int g = (int)(cy * m0 + y * m1);
    
    m0 = cx * b0 + x * b1;
    m1 = cx * b2 + x * b3;
    int b = (int)(cy * m0 + y * m1);
    UInt32 color = (a << 24) | (r << 16) | (g << 8) | b;
    return color;
}

- (void)setImage:(UIImage *)image
{
    if (image == nil)
    {
        return;
    }
    _image = image;
    self.available = [self reloadBitmapData];
}

- (RippleImageBitMap *)init
{
    self = [super init];
    if (self)
    {
        self.image = nil;
        self.replaceImage = nil;
        self.replaceRadius = 0;
        self.available = NO;
    }
    return self;
}

// 获取image的bitmap
- (BOOL)reloadBitmapData
{
    if (imageBitmapData) free(imageBitmapData);
    if (createBitmapData) free(createBitmapData);
    
    unsigned char *data = [self.image bitmapData];
    if (data == NULL)
    {
        return NO;
    }
    imageBitmapData = (UInt32 *)data;
    NSInteger w = [self.image pixw], h = [self.image pixh];
    createBitmapData = malloc(w * h * 4);
    return YES;
}

- (void)setReplaceImage:(UIImage *)replaceImage
{
    if (replaceBitmapData) free(replaceBitmapData);
    _replaceImage = replaceImage;
    replaceBitmapData = (UInt32 *)[_replaceImage bitmapData];
}

- (void)setReplaceRadius:(CGFloat)replaceRadius
{
    _replaceRadius = replaceRadius * replaceRadius;
}

- (void)dealloc
{
    if (imageBitmapData) free(imageBitmapData);
    if (createBitmapData) free(createBitmapData);
    if (replaceBitmapData) free(replaceBitmapData);
}

@end


/** 帮助类 记录波纹的参数和原位图 */
@interface RippleImageViewHelpModel : NSObject
@property (nonatomic, assign) CGPoint point;
@property (nonatomic, assign) CGFloat radius;
@property (nonatomic, assign) CGFloat wavelength;
@property (nonatomic, assign) CGFloat amplitude;
@property (nonatomic, assign) CGFloat speed;
@property (nonatomic, assign) CGFloat dx;
@property (nonatomic, assign) CGFloat dy;
@property (nonatomic, strong) RippleImageBitMap *bitmap;

- (UIImage *)rippleImageWithPhase:(CGFloat)phase;
@end

@implementation RippleImageViewHelpModel

- (UIImage *)rippleImageWithPhase:(CGFloat)phase
{
    return [self.bitmap rippleImageAtPoint:self.point wavelength:self.wavelength amplitude:self.amplitude phase:phase raduis:self.radius dx:self.dx dy:self.dy];
}

@end



@implementation UIImageView (Category)

#pragma mark - start/stop animation
- (void)startRippleAnimation:(BOOL (^)(void))block
{
    CGFloat rad = MIN(self.width, self.height) / 2;
    [self startRippleAnimationAtPoint:CGPointMake(0.5, 0.5) radius:rad callback:block];
}

- (void)startRippleAnimationAtPoint:(CGPoint)point radius:(CGFloat)radius callback:(BOOL (^)(void))block
{
    RippleImageViewHelpModel *mol = [self imageviewRippleAnimationModel];
    CGFloat len = mol ? mol.wavelength : 4;
    CGFloat amp = mol ? mol.amplitude : 15;
    [self startRippleAnimationAtPoint:point radius:radius wavelength:len amplitude:amp callback:block];
}

- (void)startRippleAnimationAtPoint:(CGPoint)point radius:(CGFloat)radius wavelength:(CGFloat)wavelength amplitude:(CGFloat)amplitude callback:(BOOL (^)(void))block
{
    RippleImageViewHelpModel *mol = [self imageviewRippleAnimationModel];
    CGFloat speed = mol ? mol.speed : 10;
    [self startRippleAnimationAtPoint:point radius:radius wavelength:wavelength amplitude:amplitude speed:speed callback:block];
}

- (void)startRippleAnimationAtPoint:(CGPoint)point radius:(CGFloat)radius wavelength:(CGFloat)wavelength amplitude:(CGFloat)amplitude speed:(CGFloat)speed callback:(BOOL (^)(void))block
{
    if (self.image == nil)
    {
        NSLog(@"请先设置image");
        return;
    }
    // 如果image尺寸与imageview 不符, 重新截图生成image
    if (!CGSizeEqualToSize(self.image.size, self.size))
    {
        self.image = [self regetSizedImage:self.image];
    }
    UIImage *replaceimage;
    RippleImageViewHelpModel *model = [self imageviewRippleAnimationModel];
    if (model != nil)
    {
        replaceimage = model.bitmap.replaceImage;
    }
    if ([self imageviewRippleAnimationStopBlock] != NULL || [self imageviewRippleAnimationDisplayLink] != nil || [self imageviewRippleAnimationModel] != nil)
    {
        [self stopRippleAnimation];
    }
    
    model = [self imageviewRippleAnimationModel];
    model.point = point;
    model.radius = radius;
    model.wavelength = wavelength;
    model.amplitude = amplitude;
    model.speed = speed > 0 ? speed : 10;
    model.dx = 1;
    model.dy = 1;
    model.bitmap.image = self.image;
    model.bitmap.replaceImage = replaceimage;
    
    //    [self resetImageviewRippleAnimationModel:model];
    [self resetImageviewRippleAnimationStopBlock:block];
    CADisplayLink *link = [CADisplayLink displayLinkWithTarget:self selector:@selector(imageviewRippleAnimationHandle)];
    [link addToRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
    link.frameInterval = 3;
    [self resetImageviewRippleAnimationDisplayLink:link];
}

- (void)stopRippleAnimation
{
    CADisplayLink *link = [self imageviewRippleAnimationDisplayLink];
    [link invalidate];
    link = nil;
    [self resetImageviewRippleAnimationModel:nil];
    [self resetImageviewRippleAnimationStopBlock:NULL];
    [self resetImageviewRippleAnimationDisplayLink:nil];
}

- (void)restartRippleAnimation
{
    CADisplayLink *link = [self imageviewRippleAnimationDisplayLink];
    if (link)
    {
        [link invalidate];
        link = nil;
    }
    link = [CADisplayLink displayLinkWithTarget:self selector:@selector(imageviewRippleAnimationHandle)];
    [link addToRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
    link.frameInterval = 3;
    [self resetImageviewRippleAnimationDisplayLink:link];
}

- (void)suspendRippleAnimation
{
    CADisplayLink *link = [self imageviewRippleAnimationDisplayLink];
    [link invalidate];
    link = nil;
}


#pragma mark - calculating handle
- (void)imageviewRippleAnimationHandle
{
    BOOL stop = [self imageviewRippleAnimationStopBlock]();
    RippleImageViewHelpModel *del = [self imageviewRippleAnimationModel];
    if (stop || del == nil)
    {
        [self stopRippleAnimation];
    }
    static CGFloat phase = 0;
    phase += del.wavelength * del.speed / 60.;
    //    if (phase > del.radius)
    //    {   // 减去过剩偏移量
    //        NSInteger num = (NSInteger)(phase / del.wavelength);
    //        phase = (num + 1) * del.wavelength - phase;
    //    }
    [self setImage:[del rippleImageWithPhase:phase]];
}

#pragma mark - user setting handle
- (void)resetRippleRadius:(CGFloat)radius
{
    RippleImageViewHelpModel *del = [self imageviewRippleAnimationModel];
    del.radius = radius;
}

- (void)resetRippleWavelength:(CGFloat)wavelength
{
    RippleImageViewHelpModel *del = [self imageviewRippleAnimationModel];
    del.wavelength = wavelength;
}

- (void)resetRippleAmplitude:(CGFloat)amplitude
{
    RippleImageViewHelpModel *del = [self imageviewRippleAnimationModel];
    del.amplitude = amplitude;
}

- (void)resetRippleSpeed:(CGFloat)speed
{
    RippleImageViewHelpModel *del = [self imageviewRippleAnimationModel];
    del.speed = speed;
}

- (void)resetRippleSlope_dx:(CGFloat)dx dy:(CGFloat)dy
{
    dx = MIN(MAX(1, dx), 10);
    dy = MIN(MAX(1, dy), 10);
    RippleImageViewHelpModel *del = [self imageviewRippleAnimationModel];
    del.dx = dx;
    del.dy = dy;
}

- (void)resetRippleCenter:(CGPoint)center
{
    center.x = MIN(MAX(0, center.x), 1);
    center.y = MIN(MAX(0, center.y), 1);
    RippleImageViewHelpModel *del = [self imageviewRippleAnimationModel];
    del.point = center;
}

- (void)resetRippleImage:(UIImage *)image
{
    if (!image) return;
    image = [self regetSizedImage:image];
    RippleImageViewHelpModel *del = [self imageviewRippleAnimationModel];
    del.bitmap.image = image;
    [self setImage:image];
}

- (UIImage *)regetSizedImage:(UIImage *)image
{
    UIImage *img = image;
    if (!CGSizeEqualToSize(image.size, self.size))
    {
        img = [UIImage imageWithImageSimple:image scaledToSize:self.size];
    }
    return img;
}

- (void)resetRippleReplaceImage:(UIImage *)image
{
    if (!image) return;
    image = [self regetSizedImage:image];
    RippleImageViewHelpModel *del = [self imageviewRippleAnimationModel];
    del.bitmap.replaceImage = image;
}

- (void)resetRippleReplaceRadius:(CGFloat)radius
{
    RippleImageViewHelpModel *del = [self imageviewRippleAnimationModel];
    del.bitmap.replaceRadius = radius;
}


#pragma mark - getters setters
- (RippleImageViewHelpModel *)imageviewRippleAnimationModel
{
    RippleImageViewHelpModel *mod = (RippleImageViewHelpModel *)objc_getAssociatedObject(self, _cmd);
    if (mod == nil)
    {
        RippleImageViewHelpModel *model = [[RippleImageViewHelpModel alloc] init];
        model.point = CGPointMake(0.5, 0.5);
        model.radius = MIN(self.width, self.height) / 2;
        model.wavelength = 4;
        model.amplitude = 15;
        model.speed = 10;
        model.dx = 1;
        model.dy = 1;
        model.bitmap = [[RippleImageBitMap alloc] init];
        model.bitmap.image = self.image;
        [self resetImageviewRippleAnimationModel:model];
    }
    return objc_getAssociatedObject(self, _cmd);
}

- (void)resetImageviewRippleAnimationModel:(RippleImageViewHelpModel *)model
{
    objc_setAssociatedObject(self, @selector(imageviewRippleAnimationModel), model, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (CADisplayLink *)imageviewRippleAnimationDisplayLink
{
    return objc_getAssociatedObject(self, _cmd);
}

- (void)resetImageviewRippleAnimationDisplayLink:(CADisplayLink *)link
{
    objc_setAssociatedObject(self, @selector(imageviewRippleAnimationDisplayLink), link, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (BOOL (^)(void))imageviewRippleAnimationStopBlock
{
    return objc_getAssociatedObject(self, _cmd);
}

- (void)resetImageviewRippleAnimationStopBlock:(BOOL (^)(void))block
{
    objc_setAssociatedObject(self, @selector(imageviewRippleAnimationStopBlock), block, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

@end
