//
//  UIView+Category.h
//  RippleImageView
//
//  Created by LY on 2016/12/9.
//  Copyright © 2016年 LY. All rights reserved.
//

#import <UIKit/UIKit.h>

typedef NS_ENUM(NSUInteger, UIViewGestureRecognizerType) {
    UIViewGestureRecognizerType_LongPress,
    UIViewGestureRecognizerType_Pan,
    UIViewGestureRecognizerType_Pinch,
    UIViewGestureRecognizerType_Rotation,
    UIViewGestureRecognizerType_ScreenEdgePan,
    UIViewGestureRecognizerType_Swipe,
    UIViewGestureRecognizerType_Tap,
};

typedef void(^UIViewGestureRecognizerBlock)(UIGestureRecognizer *gesture, UIViewGestureRecognizerType type);

typedef void(^UIViewGestureRecognizerPreset)(UIGestureRecognizer *gesture);

typedef BOOL(^UIViewGestureRecognizerDelegateBlock)(UIGestureRecognizer *gesture, UITouch *touch);

@interface UIView (Category)


/** 对view的相关属性快捷操作 */
@property (nonatomic) CGFloat x;
@property (nonatomic) CGFloat y;
@property (nonatomic) CGFloat width;
@property (nonatomic) CGFloat height;

@property (nonatomic) CGFloat left;
@property (nonatomic) CGFloat right;
@property (nonatomic) CGFloat top;
@property (nonatomic) CGFloat bottom;

@property (nonatomic) CGPoint origin;
@property (nonatomic) CGFloat originX;
@property (nonatomic) CGFloat originY;
@property (nonatomic) CGSize size;
@property (nonatomic) CGFloat centerX;
@property (nonatomic) CGFloat centerY;

/** 获取视图快照 */
- (UIImage *)screenShot;

/** 截屏 */
+ (UIImage *)snapshot;

/**
 * 添加视图手势handle 用block形式回调
 * @param   type         手势类型
 * @param   setting      对手势参数的更多设置回调
 * @param   receiveBlock 手势代理回调, 以防越权接受UIbutton/UItableview等控件的点击事件
 * @param   block        手势识别事件回调
 */
- (void)addGestureRecognizerType:(UIViewGestureRecognizerType)type moreSettingForGesture:(UIViewGestureRecognizerPreset)setting shouldReceiveTouch:(UIViewGestureRecognizerDelegateBlock)receiveBlock callback:(UIViewGestureRecognizerBlock)block;

/**
 * 按类型去除手势回调handle
 * @param   type         手势类型
 */
- (void)removeGestureRecognizerType:(UIViewGestureRecognizerType)type;

@end
