//
//  UIView+Category.m
//  RippleImageView
//
//  Created by LY on 2016/12/9.
//  Copyright © 2016年 LY. All rights reserved.
//

#import "UIView+Category.h"
#import "CG_Extention.h"
#import <objc/runtime.h>
#import "CALayer+Category.h"


@interface UIViewGestureRecognizerHelpObject : NSObject<UIGestureRecognizerDelegate>

@property (nonatomic, strong) UIGestureRecognizer *gestuer;
@property (nonatomic, assign) UIViewGestureRecognizerType type;
@property (nonatomic, copy)   UIViewGestureRecognizerBlock block;
@property (nonatomic, copy)   UIViewGestureRecognizerDelegateBlock gestureDelBlock;

@end

@implementation UIViewGestureRecognizerHelpObject

- (UIViewGestureRecognizerHelpObject *)initWithType:(UIViewGestureRecognizerType)type callback:(UIViewGestureRecognizerBlock)block gesDelBlo:(UIViewGestureRecognizerDelegateBlock)gesblo
{
    self = [super init];
    if (self)
    {
        self.type = type;
        self.block = block;
        self.gestureDelBlock = gesblo;
        [self reload];
    }
    return self;
}


- (void)reload
{
    SEL fun = @selector(gestureActionHandle:);
    switch (self.type)
    {
        case UIViewGestureRecognizerType_LongPress:
        {
            UILongPressGestureRecognizer *ges = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:fun];
            ges.numberOfTouchesRequired = 1;
            ges.numberOfTapsRequired = 1;
            self.gestuer = ges;
            break;
        }
        case UIViewGestureRecognizerType_Pan:
        {
            UIPanGestureRecognizer *ges = [[UIPanGestureRecognizer alloc] initWithTarget:self action:fun];
            ges.maximumNumberOfTouches = 3;
            ges.minimumNumberOfTouches = 1;
            self.gestuer = ges;
            break;
        }
        case UIViewGestureRecognizerType_Pinch:
        {
            UIPinchGestureRecognizer *ges = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:fun];
            self.gestuer = ges;
            break;
        }
        case UIViewGestureRecognizerType_Rotation:
        {
            UIRotationGestureRecognizer *ges = [[UIRotationGestureRecognizer alloc] initWithTarget:self action:fun];
            self.gestuer = ges;
            break;
        }
        case UIViewGestureRecognizerType_ScreenEdgePan:
        {
            UIScreenEdgePanGestureRecognizer *ges = [[UIScreenEdgePanGestureRecognizer alloc] initWithTarget:self action:fun];
            ges.maximumNumberOfTouches = 3;
            ges.minimumNumberOfTouches = 1;
            self.gestuer = ges;
            break;
        }
        case UIViewGestureRecognizerType_Swipe:
        {
            UISwipeGestureRecognizer *ges = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:fun];
            ges.numberOfTouchesRequired = 1;
            self.gestuer = ges;
            break;
        }
        case UIViewGestureRecognizerType_Tap:
        {
            UITapGestureRecognizer *ges = [[UITapGestureRecognizer alloc] initWithTarget:self action:fun];
            ges.numberOfTouchesRequired = 1;
            ges.numberOfTapsRequired = 1;
            self.gestuer = ges;
            break;
        }
        default:
            break;
    }
    self.gestuer.delegate = self;
}

- (void)gestureActionHandle:(UIGestureRecognizer *)ges
{
    self.block(self.gestuer, self.type);
}

#pragma mark - UIGestureRecognizerDelegate
- (BOOL)gestureRecognizerShouldBegin:(UIGestureRecognizer *)gestureRecognizer
{
    return YES;
}

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch
{
    if (self.gestureDelBlock != NULL)
    {
        return self.gestureDelBlock(gestureRecognizer, touch);
    }
    return YES;
}

//- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceivePress:(UIPress *)press
//{
//    if (self.gestureDelBlock != NULL)
//    {
//        return self.gestureDelBlock(gestureRecognizer, nil, press);
//    }
//    return YES;
//}

- (void)dealloc
{
    self.gestuer.delegate = nil;
    self.gestuer = nil;
    self.block = NULL;
    self.gestureDelBlock = NULL;
}

@end


@implementation UIView (Category)

- (UIImage *)screenShot
{
    return [self.layer screenShot];
}

+ (UIImage *)snapshot
{
    UIView *view = [[UIApplication sharedApplication] keyWindow];
    UIImage *image = [view screenShot];
    return image;
}

- (void)addGestureRecognizerType:(UIViewGestureRecognizerType)type moreSettingForGesture:(UIViewGestureRecognizerPreset)setting shouldReceiveTouch:(UIViewGestureRecognizerDelegateBlock)receiveBlock callback:(UIViewGestureRecognizerBlock)block
{
    [self createViewBlockGestureHelpWithType:type callback:block gestureBlock:receiveBlock];
    UIViewGestureRecognizerHelpObject *help = [self helpObjectFotType:type];
    setting(help.gestuer);
    [self addGestureRecognizer:help.gestuer];
}

- (void)removeGestureRecognizerType:(UIViewGestureRecognizerType)type
{
    switch (type)
    {
        case UIViewGestureRecognizerType_LongPress:
        {
            [self resetViewBlockGestureHelp_LongPress:nil];
            break;
        }
        case UIViewGestureRecognizerType_Pan:
        {
            [self resetViewBlockGestureHelp_Pan:nil];
            break;
        }
        case UIViewGestureRecognizerType_Pinch:
        {
            [self resetViewBlockGestureHelp_Pinch:nil];
            break;
        }
        case UIViewGestureRecognizerType_Rotation:
        {
            [self resetViewBlockGestureHelp_Rotation:nil];
            break;
        }
        case UIViewGestureRecognizerType_ScreenEdgePan:
        {
            [self resetViewBlockGestureHelp_ScreenEdgePan:nil];
            break;
        }
        case UIViewGestureRecognizerType_Swipe:
        {
            [self resetViewBlockGestureHelp_Swipe:nil];
            break;
        }
        case UIViewGestureRecognizerType_Tap:
        {
            [self resetViewBlockGestureHelp_Tap:nil];
            break;
        }
        default:
            break;
    }
}

- (void)createViewBlockGestureHelpWithType:(UIViewGestureRecognizerType)type callback:(UIViewGestureRecognizerBlock)block gestureBlock:(UIViewGestureRecognizerDelegateBlock)gesblock
{
    UIViewGestureRecognizerHelpObject *help = [[UIViewGestureRecognizerHelpObject alloc] initWithType:type callback:block gesDelBlo:gesblock];
    switch (type)
    {
        case UIViewGestureRecognizerType_LongPress:
        {
            [self resetViewBlockGestureHelp_LongPress:help];
            break;
        }
        case UIViewGestureRecognizerType_Pan:
        {
            [self resetViewBlockGestureHelp_Pan:help];
            break;
        }
        case UIViewGestureRecognizerType_Pinch:
        {
            [self resetViewBlockGestureHelp_Pinch:help];
            break;
        }
        case UIViewGestureRecognizerType_Rotation:
        {
            [self resetViewBlockGestureHelp_Rotation:help];
            break;
        }
        case UIViewGestureRecognizerType_ScreenEdgePan:
        {
            [self resetViewBlockGestureHelp_ScreenEdgePan:help];
            break;
        }
        case UIViewGestureRecognizerType_Swipe:
        {
            [self resetViewBlockGestureHelp_Swipe:help];
            break;
        }
        case UIViewGestureRecognizerType_Tap:
        {
            [self resetViewBlockGestureHelp_Tap:help];
            break;
        }
        default:
            break;
    }
}

- (UIViewGestureRecognizerHelpObject *)helpObjectFotType:(UIViewGestureRecognizerType)type
{
    UIViewGestureRecognizerHelpObject *help;
    switch (type)
    {
        case UIViewGestureRecognizerType_LongPress:
        {
            help = [self viewBlockGestureHelp_LongPress];
            break;
        }
        case UIViewGestureRecognizerType_Pan:
        {
            help = [self viewBlockGestureHelp_Pan];
            break;
        }
        case UIViewGestureRecognizerType_Pinch:
        {
            help = [self viewBlockGestureHelp_Pinch];
            break;
        }
        case UIViewGestureRecognizerType_Rotation:
        {
            help = [self viewBlockGestureHelp_Rotation];
            break;
        }
        case UIViewGestureRecognizerType_ScreenEdgePan:
        {
            help = [self viewBlockGestureHelp_ScreenEdgePan];
            break;
        }
        case UIViewGestureRecognizerType_Swipe:
        {
            help = [self viewBlockGestureHelp_Swipe];
            break;
        }
        case UIViewGestureRecognizerType_Tap:
        {
            help = [self viewBlockGestureHelp_Tap];
            break;
        }
        default:
            break;
    }
    return help;
}

- (UIViewGestureRecognizerHelpObject *)viewBlockGestureHelp_LongPress
{
    return objc_getAssociatedObject(self, _cmd);
}

- (void)resetViewBlockGestureHelp_LongPress:(UIViewGestureRecognizerHelpObject *)gesture
{
    objc_setAssociatedObject(self, @selector(viewBlockGestureHelp_LongPress), gesture, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (UIViewGestureRecognizerHelpObject *)viewBlockGestureHelp_Pan
{
    return objc_getAssociatedObject(self, _cmd);
}

- (void)resetViewBlockGestureHelp_Pan:(UIViewGestureRecognizerHelpObject *)gesture
{
    objc_setAssociatedObject(self, @selector(viewBlockGestureHelp_Pan), gesture, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (UIViewGestureRecognizerHelpObject *)viewBlockGestureHelp_Pinch
{
    return objc_getAssociatedObject(self, _cmd);
}

- (void)resetViewBlockGestureHelp_Pinch:(UIViewGestureRecognizerHelpObject *)gesture
{
    objc_setAssociatedObject(self, @selector(viewBlockGestureHelp_Pinch), gesture, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (UIViewGestureRecognizerHelpObject *)viewBlockGestureHelp_Rotation
{
    return objc_getAssociatedObject(self, _cmd);
}

- (void)resetViewBlockGestureHelp_Rotation:(UIViewGestureRecognizerHelpObject *)gesture
{
    objc_setAssociatedObject(self, @selector(viewBlockGestureHelp_Rotation), gesture, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (UIViewGestureRecognizerHelpObject *)viewBlockGestureHelp_ScreenEdgePan
{
    return objc_getAssociatedObject(self, _cmd);
}

- (void)resetViewBlockGestureHelp_ScreenEdgePan:(UIViewGestureRecognizerHelpObject *)gesture
{
    objc_setAssociatedObject(self, @selector(viewBlockGestureHelp_ScreenEdgePan), gesture, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (UIViewGestureRecognizerHelpObject *)viewBlockGestureHelp_Swipe
{
    return objc_getAssociatedObject(self, _cmd);
}

- (void)resetViewBlockGestureHelp_Swipe:(UIViewGestureRecognizerHelpObject *)gesture
{
    objc_setAssociatedObject(self, @selector(viewBlockGestureHelp_Swipe), gesture, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (UIViewGestureRecognizerHelpObject *)viewBlockGestureHelp_Tap
{
    return objc_getAssociatedObject(self, _cmd);
}

- (void)resetViewBlockGestureHelp_Tap:(UIViewGestureRecognizerHelpObject *)gesture
{
    objc_setAssociatedObject(self, @selector(viewBlockGestureHelp_Tap), gesture, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

#pragma mark - setters getters
- (void)setX:(CGFloat)x
{
    CGRect frame = self.frame;
    frame.origin.x = x;
    self.frame = frame;
}

- (CGFloat)x
{
    return self.frame.origin.x;
}

- (void)setY:(CGFloat)y
{
    CGRect frame = self.frame;
    frame.origin.y = y;
    self.frame = frame;
}

- (CGFloat)y
{
    return self.frame.origin.y;
}

- (void)setWidth:(CGFloat)width
{
    CGRect frame = self.frame;
    frame.size.width = width;
    self.frame = frame;
}

- (CGFloat)width
{
    return self.frame.size.width;
}

- (void)setHeight:(CGFloat)height
{
    CGRect frame = self.frame;
    frame.size.height = height;
    self.frame = frame;
}

- (CGFloat)height
{
    return self.frame.size.height;
}

- (void)setLeft:(CGFloat)left
{
    self.x = left;
}

- (CGFloat)left
{
    return self.x;
}

- (void)setRight:(CGFloat)right
{
    CGFloat x = right - self.width;
    self.x = x;
}

- (CGFloat)right
{
    return self.x + self.width;
}

- (void)setTop:(CGFloat)top
{
    self.y = top;
}

- (CGFloat)top
{
    return self.y;
}

- (void)setBottom:(CGFloat)bottom
{
    CGFloat y = bottom - self.height;
    self.y = y;
}

- (CGFloat)bottom
{
    return self.y + self.height;
}

- (void)setOrigin:(CGPoint)origin
{
    CGRect frame = self.frame;
    frame.origin = origin;
    self.frame = frame;
}

- (CGPoint)origin
{
    return self.frame.origin;
}

- (void)setOriginX:(CGFloat)originX
{
    self.x = originX;
}

- (CGFloat)originX
{
    return self.x;
}

- (void)setOriginY:(CGFloat)originY
{
    self.y = originY;
}

- (CGFloat)originY
{
    return self.y;
}

- (void)setSize:(CGSize)size
{
    CGRect frame = self.frame;
    frame.size = size;
    self.frame = frame;
}

- (CGSize)size
{
    return self.frame.size;
}

- (void)setCenterX:(CGFloat)centerX
{
    CGPoint center = self.center;
    center.x = centerX;
    self.center = center;
}

- (CGFloat)centerX
{
    return self.center.x;
}

- (void)setCenterY:(CGFloat)centerY
{
    CGPoint center = self.center;
    center.y = centerY;
    self.center = center;
}

- (CGFloat)centerY
{
    return self.center.y;
}

@end
